<?php

namespace Routiz\Inc\Src\Notification;

abstract class Base {

    public $user_id;
    public $userdata;
    public $meta = null;
    public $user_can_manage = false;

    public $params = [];

    function __construct( $args = [], $params = [] ) {

        if( is_array( $args ) ) {
            $args = (object) $args;
        }

        // check if user
        if( ! isset( $args->user_id ) ) {
            return;
        }

        $this->user_id = (int) $args->user_id;

        // check if user exists
        $this->userdata = get_userdata( $this->user_id );
        if( ! $this->userdata ) {
            return;
        }

        if( isset( $args->meta ) ) {
            $this->meta = is_array( $args->meta ) ? serialize( $args->meta ) : $args->meta;
        }

        if( $params ) {
            $this->params = $params;
        }

    }

    protected function get_user_email() {
        return $this->userdata->user_email;
    }

    protected function get_user_name() {
        return $this->userdata->display_name;
    }

    protected function get_user_first_name() {
        return $this->userdata->first_name;
    }

    protected function get_user_last_name() {
        return $this->userdata->last_name;
    }

    abstract public function get_id();

    abstract public function get_name();

    public function is_email_enabled() {
        return get_option( sprintf( 'rz_is_notification_email_%s', $this->get_id() ) );
    }

    public function is_email_admin_enabled() {
        return get_option( sprintf( 'rz_is_notification_email_admin_%s', $this->get_id() ) );
    }

    public function is_site_enabled() {
        return get_option( sprintf( 'rz_is_notification_site_%s', $this->get_id() ) );
    }

    public function is_webhook_enabled() {
        return get_option( sprintf( 'rz_is_notification_webhook_%s', $this->get_id() ) );
    }

    public function is_user_enabled() {
        return ! get_user_meta( $this->user_id, sprintf( 'rz_is_user_notification_%s', $this->get_id() ), true );
    }

    /*
     * send
     *
     */
    public function send() {

        $this->before_send();

        $this->send_email();
        $this->send_email_admin();
        $this->send_site();
        $this->send_webhook();

        $this->after_send();

    }

    /*
     * send email
     *
     */
    abstract public function get_email_subject();

    abstract public function get_email_admin_subject();

    abstract public function get_email_template();

    abstract public function get_email_admin_template();

    public function get_email_template_from_option() {
        return get_option( sprintf('rz_notification_email_template_%s', $this->get_id() ) );
    }

    public function get_email_admin_template_from_option() {
        return get_option( sprintf('rz_notification_email_template_admin_%s', $this->get_id() ) );
    }

    public function before_send() {
        // ..
    }

    public function after_send() {
        // ..
    }

    public function send_email() {

        // check if enabled by site
        if( ! $this->is_email_enabled() ) {
            return;
        }

        // check if enabled by the user
        if( ! $this->is_user_enabled() ) {
            return;
        }

        if( empty( $message = $this->get_email_template_from_option() ) ) {
            $message = $this->get_email_template();
        }

        // replace message fields
        $message = str_replace('{email}', $this->get_user_email(), $message );
        $message = str_replace('{name}', $this->get_user_name(), $message );

        // email
        wp_mail(
            $this->get_user_email(),
            $this->get_email_subject(),
            $message,
            [
                'MIME-Version: 1.0',
                'Content-Type: text/html; charset=UTF-8'
            ]
        );

    }

    public function send_email_admin() {

        // check if enabled by site
        if( ! $this->is_email_admin_enabled() ) {
            return;
        }

        if( empty( $message = $this->get_email_admin_template_from_option() ) ) {
            $message = $this->get_email_admin_template();
        }

        // replace message fields
        $message = str_replace('{email}', $this->get_user_email(), $message );
        $message = str_replace('{name}', $this->get_user_name(), $message );

        // email
        wp_mail(
            get_option('admin_email'),
            $this->get_email_admin_subject(),
            $message,
            [
                'MIME-Version: 1.0',
                'Content-Type: text/html; charset=UTF-8'
            ]
        );

    }

    /*
     * send site
     *
     */
    abstract public function get_site_message();

    // get last repeated site notifications in 24 hours
    public function check_last() {

        global $wpdb;

        $results = $wpdb->get_row( $wpdb->prepare("
            SELECT *
            FROM {$wpdb->prefix}routiz_notifications
            WHERE user_id = %d
            AND code = %s
            AND active = 1
            AND meta = %s
            AND created_at >= date( now() ) - 1
        ", $this->user_id, $this->get_id(), $this->meta ), OBJECT );

        return !! $results;

    }

    public function get_site_icon() {
        return 'fas fa-magic';
    }

    public function get_site_url() {
        return null;
    }

    public function get_site() {

        return [
            'code'  => $this->get_id(),
            'text'  => str_replace('{name}', $this->get_user_first_name(), $this->get_site_message() ),
            'icon'  => $this->get_site_icon(),
            'url'   => $this->get_site_url(),
        ];

    }

    // send
    public function send_site() {

        // check if enabled by site
        if( ! $this->is_site_enabled() ) {
            return;
        }

        // check if enabled by the user
        if( ! $this->is_user_enabled() ) {
            return;
        }

        global $wpdb;

        // check if repeated message
        if( $this->check_last() ) {
            return;
        }

        // insert into db
        return $wpdb->insert( $wpdb->prefix . 'routiz_notifications', [
            'user_id' => $this->user_id,
            'code' => $this->get_id(),
            'active' => 1,
            'meta' => $this->meta
        ]);

    }

    /*
     * send webhook
     *
     */
    public function get_webhook_url() {
        return get_option( sprintf( 'rz_notification_webhook_url_%s', $this->get_id() ) );
    }

    public function send_webhook() {

        // check if enabled by site
        if( ! $this->is_webhook_enabled() ) {
            return;
        }

        // check if enabled by the user
        if( ! $this->is_user_enabled() ) {
            return;
        }

        $webhook_url = $this->get_webhook_url();

        // check url
        if( empty( $webhook_url ) or filter_var( $webhook_url, FILTER_VALIDATE_URL ) === false ) {
            return;
        }

        $data = [
            'email' => $this->get_user_email(),
            'name' => $this->get_user_name(),
            'first_name' => $this->get_user_first_name(),
            'last_name' => $this->get_user_last_name(),
            'billing_first_name' => get_user_meta( $this->user_id, 'billing_first_name', true ),
            'billing_last_name' => get_user_meta( $this->user_id, 'billing_last_name', true ),
            'billing_email' => get_user_meta( $this->user_id, 'billing_email', true ),
            'billing_phone' => get_user_meta( $this->user_id, 'billing_phone', true ),
            'billing_country' => get_user_meta( $this->user_id, 'billing_country', true ),
            'billing_city' => get_user_meta( $this->user_id, 'billing_city', true ),
            'billing_postcode' => get_user_meta( $this->user_id, 'billing_postcode', true ),
        ];

        $ch = curl_init( $webhook_url );
        curl_setopt( $ch, CURLOPT_HTTPHEADER, [
            'Content-type: application/json'
        ]);
        curl_setopt( $ch, CURLOPT_POST, 1);
        curl_setopt( $ch, CURLOPT_POSTFIELDS, json_encode( $data, JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE ));
        curl_setopt( $ch, CURLOPT_FOLLOWLOCATION, 1);
        curl_setopt( $ch, CURLOPT_HEADER, 0);
        curl_setopt( $ch, CURLOPT_RETURNTRANSFER, 1);

        $response = curl_exec( $ch );

        curl_close( $ch );

    }

}
